class Battle_Set:
	def count_party():
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st		
		party_num = 0
		for i in range(st.PARTY_NUM):
			if co_v.party[i] != 0:
				party_num += 1
		return party_num
	
	def select_ene():
		import random
		from ..Basic.Settings import Settings as st
		ene_num = random.randint(0,st.IZMON_P_LEN)
		ene = st.IZMON_P[ene_num]
		return ene
	
	def set(ene : int, typ : int): #各種値を設定
		from .Battle_Caluculate import Battle_Caluculate as bat_ca
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st	
		from ..Gym import Gym as gym
		import statistics
	
		tech = [["", "", "", "", "", ""],
		  		["", "", "", "", "", ""],
				["", "", "", "", "", ""],
				["", "", "", "", "", ""],
				[0, 0, 0, 0, 0, 0],
				[0, 0, 0, 0, 0, 0],
				[0, 0, 0, 0, 0, 0],
				[0, 0, 0, 0, 0, 0],
				[0, 0, 0, 0, 0, 0],
				[0, 0, 0, 0, 0, 0],
				[0, 0, 0, 0, 0, 0],
				[0, 0, 0, 0, 0, 0]]
		damage_all = [[0, 0, 0, 0, 0, 0],
					  [0, 0, 0, 0, 0, 0],
					  [0, 0, 0, 0, 0, 0],
					  [0, 0, 0, 0, 0, 0]]
		data = []
		
		IZ_SET = st.IZUMON_SETTING

		for i in range(st.PARTY_NUM):
			me_num = co_v.party[i]
			if me_num == 0:
				me_com = []
			else:
				if typ == 1:
					hp = gym.hp_me[i]
				else:
					hp = bat_ca.set_hp(me_num, co_v.level[me_num])
				me_com = [IZ_SET[me_num][0], IZ_SET[me_num][1], hp, st.SPETIAL_NUM, co_v.level[me_num]]
			data.append(me_com)
			tech = bat_ca.store_tech(tech, me_num, ene, i)
		me_level = []
		for i in range(st.PARTY_NUM):
			me_num = co_v.party[i]
			if me_num == 0:
				continue
			else:
				me_level.append(co_v.level[me_num])
		if typ == 0:
			ene_level = int(statistics.median(me_level))
		else:
			ene_level = st.LEVEL_MAX[sum(co_v.badges) + 1]

		ene_com = [IZ_SET[ene][0], IZ_SET[ene][1], bat_ca.set_hp(ene, ene_level), st.SPETIAL_NUM, ene_level]
		data.append(ene_com)

		for i in range(st.PARTY_NUM):
			if co_v.party[i] != 0:
				tech, damage_all = bat_ca.calculate_damage(damage_all, tech, co_v.party[i], ene, i, ene_level)
		
		return data, damage_all, tech

	def set_ene_level(typ : int):
		from ..Basic.Settings import Settings as st
		from ..Basic.Common_Variable import Common_Variable as co_v
		import statistics

		me_level = []
		for i in range(st.PARTY_NUM):
			me_num = co_v.party[i]
			if me_num == 0:
				continue
			else:
				me_level.append(co_v.level[me_num])
		if typ == 0:
			ene_level = int(statistics.median(me_level))
			print(ene_level)
		else:
			ene_level = st.LEVEL_MAX[sum(co_v.badges) + 1]
		return ene_level


class End:
	from ..Basic.Common_Function import Common_Function as co_f
	from ..Basic.Settings import Settings as st
	
	def save(self): #データをセーブする
		from ..Save_Code import Encode
		while True:
			ty = self.co_f.typing("データをセーブしますか？ 1:する 0:しない", 1, 2)
			if ty == 1:
				code = Encode.encode()
				print("プレイヤー名と共に保管してください。")
				print("コード：", code)
				break
			else:
				break
		return


def main():
	from Izmon.Class.Basic.Common_Function import Common_Function as co_f
	from Izmon.Class.Gym import Gym
	from Izmon.Class.Dictionary import Dictionary as dic
	from Izmon.Class.Events.Events_Latest import Events_Latest as ev
	from Izmon.Class.Start_End.Start import Start
	from Izmon.Class.Start_End.End import End
	from Izmon.Class.Set import Set
	from Izmon.Class.Battle.Battle_Set import Battle_Set as bat_s
	from Izmon.Class.Battle.Battle import Battle as ba
	
	gym = Gym(); start = Start(); end = End(); set = Set()
	start.set_up()
	while True:
		action = co_f.typing("どうしますか? 0:バトルする 1:ジムに行く 2:イズモン辞典 3:イベント 4:設定 5:やめる", 1, 6)
		if action == 0:
			enemy = bat_s.select_ene()
			ba.battle(enemy, 0)
		elif action == 1:
			gym.gym_main()
		elif action == 2:
			dic.dic(0)
		elif action == 3:
			ev.events()
		elif action == 4:
			set.main()
		elif action ==5:
			break
	end.save()



class Events_Latest:
    def events():
        from ..Basic.Settings import Settings as st
        from ..Basic.Common_Function import Common_Function as co_f
        print("=" * st.SEPALATE_LEN)
        print("現在開催中のイベントはありません")
        print("=" * st.SEPALATE_LEN)
        return


class Decode:
	def decode(code : str, typ : int):
		import hashlib
		from ..Basic.Common_Variable import Common_Variable as co_v
		from ..Basic.Settings import Settings as st
		
		player_hash = str(int(hashlib.sha256(co_v.player_name.encode("utf-8")).hexdigest(), 16))
		code_10 = str(int(code, 16))
		if int(str(code_10)[:-1]) % int(player_hash[21]) != int(str(code_10)[-1]):
			return 0
		code_10_str = str(code_10)
		if code_10_str[-21:-1] != player_hash[:20]:
			return 0
		data = format(int(code_10_str[:-21]), str(st.IZ_NUM) + 'b')
		if typ == 0:
			for i in range(100):
				if i >= st.IZ_NUM:
					continue
				else:
					co_v.fellows[i] = int(data[i])
			return 1
		else:
			return data


class Encode:
	def encode(data : str = ""):
		from ..Basic.Settings import Settings as st
		from ..Basic.Common_Variable import Common_Variable as co_v
		import hashlib
		if data == "":
			for i in range(100):
				if i >= st.IZ_NUM:
					data += "0"
				else:
					data += str(co_v.fellows[i])
		data_10 = str(int(data, 2))
		player_hash = str(int(hashlib.sha256(co_v.player_name.encode("utf-8")).hexdigest(), 16))
		data_hash_add = data_10 + player_hash[:20]
		check_digit = int(data_hash_add) % int(player_hash[21])
		data_check_add = int(data_hash_add + str(check_digit))
		code_16 = format(data_check_add, "0x")
		return code_16